@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +14,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -156,7 +154,8 @@ filter_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 					}
 				
 					/* go to the next line */
-					for ( ; inptr < inend && *inptr != '\n'; inptr++);
+					while (inptr < inend && *inptr != '\n')
+						inptr++;
 					
 					if (inptr < inend)
 						inptr++;
@@ -323,9 +322,9 @@ g_mime_filter_yenc_set_crc (GMimeFilterYenc *yenc, guint32 crc)
  * g_mime_filter_yenc_get_part:
  * @yenc: yEnc filter
  *
- * Gets the part id of the current decoded yEnc stream or -1 on fail.
+ * Gets the part id of the current decoded yEnc stream or %-1 on fail.
  *
- * Returns the part id of the current decoded yEnc stream or -1 on
+ * Returns the part id of the current decoded yEnc stream or %-1 on
  * fail.
  **/
 int
@@ -427,10 +426,10 @@ static const int yenc_crc_table[256] = {
  * @inlen pointed to by @in and writes to @out. Assumes the =ybegin
  * and =ypart lines have already been stripped off.
  *
- * To get the crc32 value of the part, use GMIME_YENCODE_CRC_FINAL
+ * To get the crc32 value of the part, use #GMIME_YENCODE_CRC_FINAL
  * (@pcrc). If there are more parts, you should reuse @crc without
  * re-initializing. Once all parts have been decoded, you may get the
- * combined crc32 value of all the parts using GMIME_YENCODE_CRC_FINAL
+ * combined crc32 value of all the parts using #GMIME_YENCODE_CRC_FINAL
  * (@crc).
  *
  * Returns the number of bytes decoded.
