@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -25,8 +22,6 @@
 #include <config.h>
 #endif
 
-#define _POSIX_SOURCE
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,6 +36,7 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
+#include <poll.h>
 
 #include "gmime-gpg-context.h"
 #include "gmime-filter-charset.h"
@@ -166,20 +162,20 @@ gpg_hash_id (GMimeCipherContext *ctx, const char *hash)
 	if (hash == NULL)
 		return GMIME_CIPHER_HASH_DEFAULT;
 	
-	if (!strcasecmp (hash, "pgp-"))
+	if (!g_ascii_strcasecmp (hash, "pgp-"))
 		hash += 4;
 	
-	if (!strcasecmp (hash, "md2"))
+	if (!g_ascii_strcasecmp (hash, "md2"))
 		return GMIME_CIPHER_HASH_MD2;
-	else if (!strcasecmp (hash, "md5"))
+	else if (!g_ascii_strcasecmp (hash, "md5"))
 		return GMIME_CIPHER_HASH_MD5;
-	else if (!strcasecmp (hash, "sha1"))
+	else if (!g_ascii_strcasecmp (hash, "sha1"))
 		return GMIME_CIPHER_HASH_SHA1;
-	else if (!strcasecmp (hash, "ripemd160"))
+	else if (!g_ascii_strcasecmp (hash, "ripemd160"))
 		return GMIME_CIPHER_HASH_RIPEMD160;
-	else if (!strcasecmp (hash, "tiger192"))
+	else if (!g_ascii_strcasecmp (hash, "tiger192"))
 		return GMIME_CIPHER_HASH_TIGER192;
-	else if (!strcasecmp (hash, "haval-5-160"))
+	else if (!g_ascii_strcasecmp (hash, "haval-5-160"))
 		return GMIME_CIPHER_HASH_HAVAL5160;
 	
 	return GMIME_CIPHER_HASH_DEFAULT;
@@ -334,7 +330,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	stream = g_mime_stream_mem_new ();
 	gpg->diag = GMIME_STREAM_MEM (stream)->buffer;
 	charset = g_mime_locale_charset ();
-	if (strcasecmp (charset, "UTF-8") != 0) {
+	if (g_ascii_strcasecmp (charset, "UTF-8") != 0) {
 		GMimeStream *fstream;
 		GMimeFilter *filter;
 		
@@ -1104,62 +1100,61 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 	gpg->statusleft -= len;                                           \
 } G_STMT_END
 
+enum {
+	GPG_STDIN_FD,
+	GPG_STDOUT_FD,
+	GPG_STDERR_FD,
+	GPG_STATUS_FD,
+	GPG_PASSWD_FD,
+	GPG_N_FDS
+};
+
 static int
 gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 {
-	fd_set rdset, wrset, *wrsetp = NULL;
 	const char *diagnostics, *mode;
-	struct timeval timeout;
+	struct pollfd pfds[GPG_N_FDS];
 	int ready, save;
-	int maxfd = 0;
+	nfds_t n;
 	
- retry:
-	FD_ZERO (&rdset);
+	for (n = 0; n < GPG_N_FDS; n++) {
+		pfds[n].events = 0;
+		pfds[n].fd = -1;
+	}
 	
 	if (!gpg->seen_eof1) {
-		FD_SET (gpg->stdout_fd, &rdset);
-		maxfd = MAX (maxfd, gpg->stdout_fd);
+		pfds[GPG_STDOUT_FD].fd = gpg->stdout_fd;
+		pfds[GPG_STDOUT_FD].events = POLLIN;
 	}
 	
 	if (!gpg->seen_eof2) {
-		FD_SET (gpg->stderr_fd, &rdset);
-		maxfd = MAX (maxfd, gpg->stderr_fd);
+		pfds[GPG_STDERR_FD].fd = gpg->stderr_fd;
+		pfds[GPG_STDERR_FD].events = POLLIN;
 	}
 	
 	if (!gpg->complete) {
-		FD_SET (gpg->status_fd, &rdset);
-		maxfd = MAX (maxfd, gpg->status_fd);
+		pfds[GPG_STATUS_FD].fd = gpg->status_fd;
+		pfds[GPG_STATUS_FD].events = POLLIN;
 	}
 	
-	if (gpg->stdin_fd != -1 || gpg->passwd_fd != -1) {
-		FD_ZERO (&wrset);
-		if (gpg->istream && gpg->stdin_fd != -1) {
-			FD_SET (gpg->stdin_fd, &wrset);
-			maxfd = MAX (maxfd, gpg->stdin_fd);
-		}
-		if (gpg->passwd_fd != -1) {
-			FD_SET (gpg->passwd_fd, &wrset);
-			maxfd = MAX (maxfd, gpg->passwd_fd);
-		}
-		
-		wrsetp = &wrset;
-	}
+	pfds[GPG_STDIN_FD].fd = gpg->stdin_fd;
+	pfds[GPG_STDIN_FD].events = POLLOUT;
 	
-	g_assert (maxfd > 0);
+	pfds[GPG_PASSWD_FD].fd = gpg->passwd_fd;
+	pfds[GPG_PASSWD_FD].events = POLLOUT;
 	
-	timeout.tv_sec = 10; /* timeout in seconds */
-	timeout.tv_usec = 0;
+	do {
+		for (n = 0; n < GPG_N_FDS; n++)
+			pfds[n].revents = 0;
+		ready = poll (pfds, GPG_N_FDS, 10 * 1000);
+	} while (ready == -1 && errno == EINTR);
 	
-	if ((ready = select (maxfd + 1, &rdset, wrsetp, NULL, &timeout)) == 0)
+	if (ready == -1) {
+		d(printf ("poll() failed: %s\n", g_strerror (errno)));
+		goto exception;
+	} else if (ready == 0) {
+		/* timed out */
 		return 0;
-	
-	if (ready < 0) {
-		if (errno == EINTR)
-			goto retry;
-		
-		d(printf ("select() failed: %s\n", g_strerror (errno)));
-		
-		return -1;
 	}
 	
 	/* Test each and every file descriptor to see if it's 'ready',
@@ -1168,7 +1163,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 	   can to all of them. If one fails along the way, return
 	   -1. */
 	
-	if (FD_ISSET (gpg->status_fd, &rdset)) {
+	if (pfds[GPG_STATUS_FD].revents & (POLLIN | POLLHUP)) {
 		/* read the status message and decide what to do... */
 		char buffer[4096];
 		ssize_t nread;
@@ -1178,6 +1173,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		do {
 			nread = read (gpg->status_fd, buffer, sizeof (buffer));
 		} while (nread == -1 && (errno == EINTR || errno == EAGAIN));
+		
 		if (nread == -1)
 			goto exception;
 		
@@ -1190,7 +1186,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		}
 	}
 	
-	if (FD_ISSET (gpg->stdout_fd, &rdset) && gpg->ostream) {
+	if ((pfds[GPG_STDOUT_FD].revents & (POLLIN | POLLHUP)) && gpg->ostream) {
 		char buffer[4096];
 		ssize_t nread;
 		
@@ -1210,7 +1206,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		}
 	}
 	
-	if (FD_ISSET (gpg->stderr_fd, &rdset)) {
+	if (pfds[GPG_STDERR_FD].revents & (POLLIN | POLLHUP)) {
 		char buffer[4096];
 		ssize_t nread;
 		
@@ -1229,7 +1225,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		}
 	}
 	
-	if (wrsetp && gpg->passwd_fd != -1 && FD_ISSET (gpg->passwd_fd, &wrset) && gpg->need_passwd && gpg->send_passwd) {
+	if ((pfds[GPG_PASSWD_FD].revents & (POLLOUT | POLLHUP)) && gpg->need_passwd && gpg->send_passwd) {
 		ssize_t w, nwritten = 0;
 		size_t n;
 		
@@ -1257,7 +1253,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		gpg->send_passwd = FALSE;
 	}
 	
-	if (gpg->istream && wrsetp && gpg->stdin_fd != -1 && FD_ISSET (gpg->stdin_fd, &wrset)) {
+	if ((pfds[GPG_STDIN_FD].revents & (POLLOUT | POLLHUP)) && gpg->istream) {
 		char buffer[4096];
 		ssize_t nread;
 		
@@ -1327,7 +1323,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 			     diagnostics);
 	} else {
 		g_set_error (err, GMIME_ERROR, errno,
-			     _("Failed to %s vua GnuPG: %s\n"),
+			     _("Failed to %s via GnuPG: %s\n"),
 			     mode, g_strerror (errno));
 	}
 	
@@ -1344,11 +1340,9 @@ gpg_ctx_op_complete (struct _GpgCtx *gpg)
 static gboolean
 gpg_ctx_op_exited (struct _GpgCtx *gpg)
 {
-	pid_t retval;
 	int status;
 	
-	retval = waitpid (gpg->pid, &status, WNOHANG);
-	if (retval == gpg->pid) {
+	if (waitpid (gpg->pid, &status, WNOHANG) == gpg->pid) {
 		gpg->exit_status = status;
 		gpg->exited = TRUE;
 		return TRUE;
@@ -1361,7 +1355,6 @@ gpg_ctx_op_exited (struct _GpgCtx *gpg)
 static void
 gpg_ctx_op_cancel (struct _GpgCtx *gpg)
 {
-	pid_t retval;
 	int status;
 	
 	if (gpg->exited)
@@ -1369,8 +1362,7 @@ gpg_ctx_op_cancel (struct _GpgCtx *gpg)
 	
 	kill (gpg->pid, SIGTERM);
 	sleep (1);
-	retval = waitpid (gpg->pid, &status, WNOHANG);
-	if (retval == (pid_t) 0) {
+	if (waitpid (gpg->pid, &status, WNOHANG) == 0) {
 		/* no more mr nice guy... */
 		kill (gpg->pid, SIGKILL);
 		sleep (1);
@@ -1483,18 +1475,15 @@ swrite (GMimeStream *istream)
 	char *template;
 	int fd, ret;
 	
-	template = g_strdup ("/tmp/gmime-pgp.XXXXXX");
-	fd = mkstemp (template);
-	if (fd == -1) {
+	template = g_build_filename (g_get_tmp_dir (), "gmime-pgp.XXXXXX", NULL);
+	if ((fd = mkstemp (template)) == -1) {
 		g_free (template);
 		return NULL;
 	}
 	
 	ostream = g_mime_stream_fs_new (fd);
-	ret = g_mime_stream_write_to_stream (istream, ostream);
-	if (ret != -1) {
-		ret = g_mime_stream_flush (ostream);
-		if (ret != -1)
+	if ((ret = g_mime_stream_write_to_stream (istream, ostream)) != -1) {
+		if ((ret = g_mime_stream_flush (ostream)) != -1)
 			ret = g_mime_stream_close (ostream);
 	}
 	g_object_unref (ostream);
@@ -1523,8 +1512,7 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	if (sigstream != NULL) {
 		/* We are going to verify a detached signature so save
 		   the signature to a temp file. */
-		sigfile = swrite (sigstream);
-		if (sigfile == NULL) {
+		if (!(sigfile = swrite (sigstream))) {
 			g_set_error (err, GMIME_ERROR, errno,
 				     _("Cannot verify message signature: "
 				       "could not create temp file: %s"),
