@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,20 +14,17 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+
 #ifndef __GMIME_HEADER_H__
 #define __GMIME_HEADER_H__
 
 #include <glib.h>
 #include <gmime/gmime-stream.h>
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 typedef struct _GMimeHeader GMimeHeader;
 
@@ -44,6 +39,8 @@ void g_mime_header_set (GMimeHeader *header, const char *name, const char *value
 
 void g_mime_header_add (GMimeHeader *header, const char *name, const char *value);
 
+void g_mime_header_prepend (GMimeHeader *header, const char *name, const char *value);
+
 const char *g_mime_header_get (const GMimeHeader *header, const char *name);
 
 void g_mime_header_remove (GMimeHeader *header, const char *name);
@@ -61,8 +58,6 @@ void g_mime_header_register_writer (GMimeHeader *header, const char *name, GMime
 void g_mime_header_set_raw (GMimeHeader *header, const char *raw);
 gboolean g_mime_header_has_raw (GMimeHeader *header);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GMIME_HEADER_H__ */
