@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -38,7 +35,7 @@
 #include "gmime-parser.h"
 #include "gmime-part.h"
 
-#define d(x) x
+#define d(x)
 
 /* GObject class methods */
 static void g_mime_multipart_signed_class_init (GMimeMultipartSignedClass *klass);
@@ -261,7 +258,7 @@ sign_prepare (GMimeObject *mime_part)
  * successful, the signed #GMimeObject is set as the signed part of
  * the multipart/signed object @mps.
  *
- * Returns 0 on success or -1 on fail. If the signing fails, an
+ * Returns %0 on success or %-1 on fail. If the signing fails, an
  * exception will be set on @err to provide information as to why the
  * failure occured.
  **/
@@ -403,12 +400,17 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), NULL);
 	g_return_val_if_fail (ctx->sign_protocol != NULL, NULL);
 	
+	if (g_mime_multipart_get_number ((GMimeMultipart *) mps) != 2) {
+		/* FIXME: set an exception */
+		return NULL;
+	}
+	
 	protocol = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mps), "protocol");
 	micalg = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mps), "micalg");
 	
 	if (protocol) {
 		/* make sure the protocol matches the cipher sign protocol */
-		if (strcasecmp (ctx->sign_protocol, protocol) != 0)
+		if (g_ascii_strcasecmp (ctx->sign_protocol, protocol) != 0)
 			return NULL;
 	} else {
 		/* *shrug* - I guess just go on as if they match? */
@@ -419,7 +421,7 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	
 	/* make sure the protocol matches the signature content-type */
 	content_type = g_mime_content_type_to_string (signature->content_type);
-	if (strcasecmp (content_type, protocol) != 0) {
+	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
 		g_object_unref (signature);
 		g_free (content_type);
 		
