@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -57,8 +54,11 @@ static void set_content_type (GMimeObject *object, GMimeContentType *content_typ
 static char *get_headers (GMimeObject *object);
 static ssize_t write_to_stream (GMimeObject *object, GMimeStream *stream);
 
+static ssize_t write_content_type (GMimeStream *stream, const char *name, const char *value);
+
 static void type_registry_init (void);
 
+
 static GHashTable *type_hash = NULL;
 
 static GObjectClass *parent_class = NULL;
@@ -116,6 +116,8 @@ g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass)
 	object->content_type = NULL;
 	object->headers = g_mime_header_new ();
 	object->content_id = NULL;
+	
+	g_mime_header_register_writer (object->headers, "Content-Type", write_content_type);
 }
 
 static void
@@ -135,6 +137,33 @@ g_mime_object_finalize (GObject *object)
 }
 
 
+static ssize_t
+write_content_type (GMimeStream *stream, const char *name, const char *value)
+{
+	GMimeContentType *content_type;
+	ssize_t nwritten;
+	GString *out;
+	char *val;
+	
+	out = g_string_new ("");
+	g_string_printf (out, "%s: ", name);
+	
+	content_type = g_mime_content_type_new_from_string (value);
+	
+	val = g_mime_content_type_to_string (content_type);
+	g_string_append (out, val);
+	g_free (val);
+	
+	g_mime_param_write_to_string (content_type->params, TRUE, out);
+	g_mime_content_type_destroy (content_type);
+	
+	nwritten = g_mime_stream_write (stream, out->str, out->len);
+	g_string_free (out, TRUE);
+	
+	return nwritten;
+}
+
+
 /**
  * g_mime_object_ref:
  * @object: mime object
@@ -193,8 +222,7 @@ g_mime_object_register_type (const char *type, const char *subtype, GType object
 	
 	type_registry_init ();
 	
-	bucket = g_hash_table_lookup (type_hash, type);
-	if (!bucket) {
+	if (!(bucket = g_hash_table_lookup (type_hash, type))) {
 		bucket = g_new (struct _type_bucket, 1);
 		bucket->type = g_strdup (type);
 		bucket->object_type = *type == '*' ? object_type : 0;
@@ -243,21 +271,19 @@ g_mime_object_new_type (const char *type, const char *subtype)
 	
 	type_registry_init ();
 	
-	bucket = g_hash_table_lookup (type_hash, type);
-	if (!bucket) {
-		bucket = g_hash_table_lookup (type_hash, "*");
-		obj_type = bucket ? bucket->object_type : 0;
-	} else {
+	if ((bucket = g_hash_table_lookup (type_hash, type))) {
 		if (!(sub = g_hash_table_lookup (bucket->subtype_hash, subtype)))
 			sub = g_hash_table_lookup (bucket->subtype_hash, "*");
 		
 		obj_type = sub ? sub->object_type : 0;
+	} else {
+		bucket = g_hash_table_lookup (type_hash, "*");
+		obj_type = bucket ? bucket->object_type : 0;
 	}
 	
 	if (!obj_type) {
 		/* use the default mime object */
-		bucket = g_hash_table_lookup (type_hash, "*");
-		if (bucket) {
+		if ((bucket = g_hash_table_lookup (type_hash, "*"))) {
 			sub = g_hash_table_lookup (bucket->subtype_hash, "*");
 			obj_type = sub ? sub->object_type : 0;
 		}
@@ -290,8 +316,7 @@ sync_content_type (GMimeObject *object)
 	g_string_append (string, type);
 	g_free (type);
 	
-	params = content_type->params;
-	if (params)
+	if ((params = content_type->params))
 		g_mime_param_write_to_string (params, FALSE, string);
 	
 	p = string->str;
@@ -443,38 +468,40 @@ static char *headers[] = {
 	NULL
 };
 
-static void
+static gboolean
 process_header (GMimeObject *object, const char *header, const char *value)
 {
 	GMimeContentType *content_type;
 	int i;
 	
 	for (i = 0; headers[i]; i++) {
-		if (!strcasecmp (headers[i], header))
+		if (!g_ascii_strcasecmp (headers[i], header))
 			break;
 	}
 	
 	switch (i) {
 	case HEADER_CONTENT_TYPE:
 		content_type = g_mime_content_type_new_from_string (value);
-		if (object->content_type)
-			g_mime_content_type_destroy (object->content_type);
-		object->content_type = content_type;
+		g_mime_object_set_content_type (object, content_type);
 		break;
 	case HEADER_CONTENT_ID:
 		g_free (object->content_id);
 		object->content_id = g_mime_utils_decode_message_id (value);
 		break;
 	default:
-		break;
+		return FALSE;
 	}
+	
+	g_mime_header_set (object->headers, header, value);
+	
+	return TRUE;
 }
 
 static void
 add_header (GMimeObject *object, const char *header, const char *value)
 {
-	process_header (object, header, value);
-	g_mime_header_add (object->headers, header, value);
+	if (!process_header (object, header, value))
+		g_mime_header_add (object->headers, header, value);
 }
 
 
@@ -500,8 +527,8 @@ g_mime_object_add_header (GMimeObject *object, const char *header, const char *v
 static void
 set_header (GMimeObject *object, const char *header, const char *value)
 {
-	process_header (object, header, value);
-	g_mime_header_set (object->headers, header, value);
+	if (!process_header (object, header, value))
+		g_mime_header_set (object->headers, header, value);
 }
 
 
@@ -663,7 +690,10 @@ g_mime_object_to_string (GMimeObject *object)
 static void
 subtype_bucket_foreach (gpointer key, gpointer value, gpointer user_data)
 {
-	g_free (key);
+	struct _subtype_bucket *bucket = value;
+	
+	g_free (bucket->subtype);
+	g_free (bucket);
 }
 
 static void
