@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -128,29 +125,47 @@ static char *
 decode_quoted_string (const char **in)
 {
 	const char *start, *inptr = *in;
-	char *out = NULL;
+	char *outptr, *out = NULL;
+	gboolean unescape = FALSE;
 	
 	decode_lwsp (&inptr);
-	if (*inptr == '"') {
-		start = inptr++;
-		
-		while (*inptr && *inptr != '"') {
-			if (*inptr++ == '\\')
-				inptr++;
-		}
-		
-		if (*inptr == '"') {
-			start++;
-			out = g_strndup (start, (unsigned int) (inptr - start));
+	
+	if (*inptr != '"') {
+		*in = inptr;
+		return NULL;
+	}
+	
+	start = inptr++;
+	
+	while (*inptr && *inptr != '"') {
+		if (*inptr++ == '\\') {
+			unescape = TRUE;
 			inptr++;
-		} else {
-			/* string wasn't properly quoted */
-			out = g_strndup (start, (unsigned int) (inptr - start));
 		}
 	}
 	
+	if (*inptr == '"') {
+		start++;
+		out = g_strndup (start, (unsigned int) (inptr - start));
+		inptr++;
+	} else {
+		/* string wasn't properly quoted */
+		out = g_strndup (start, (unsigned int) (inptr - start));
+	}
+	
 	*in = inptr;
 	
+	if (unescape) {
+		inptr = outptr = out;
+		while (*inptr) {
+			if (*inptr == '\\')
+				inptr++;
+			*outptr++ = *inptr++;
+		}
+		
+		*outptr = '\0';
+	}
+	
 	return out;
 }
 
@@ -408,7 +423,7 @@ charset_convert (const char *charset, char *in, size_t inlen)
 	char *result = NULL;
 	iconv_t cd;
 	
-	if (!charset || !strcasecmp (charset, "UTF-8") || !strcasecmp (charset, "us-ascii")) {
+	if (!charset || !g_ascii_strcasecmp (charset, "UTF-8") || !g_ascii_strcasecmp (charset, "us-ascii")) {
 		/* we shouldn't need any charset conversion here... */
 		if (g_utf8_validate (in, inlen, NULL))
 			return in;
@@ -729,7 +744,7 @@ encode_param (const unsigned char *in, gboolean *encoded)
 	if (!charset)
 		charset = "iso-8859-1";
 	
-	if (strcasecmp (charset, "UTF-8") != 0)
+	if (g_ascii_strcasecmp (charset, "UTF-8") != 0)
 		cd = g_mime_iconv_open (charset, "UTF-8");
 	
 	if (cd != (iconv_t) -1) {
