@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -213,8 +210,7 @@ stream_close (GMimeStream *stream)
 	}
 	
 	if (mstream->owner && mstream->fd != -1) {
-		ret = close (mstream->fd);
-		if (ret != -1)
+		if ((ret = close (mstream->fd)) != -1)
 			mstream->fd = -1;
 	}
 	
@@ -236,9 +232,9 @@ stream_reset (GMimeStream *stream)
 {
 	GMimeStreamMmap *mstream = (GMimeStreamMmap *) stream;
 	
-	g_return_val_if_fail (mstream->fd != -1, -1);
+	if (mstream->fd == -1)
+		return -1;
 	
-	stream->position = stream->bound_start;
 	mstream->eos = FALSE;
 	
 	return 0;
@@ -274,11 +270,16 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 		break;
 	}
 	
-	if (stream->bound_end != -1)
-		real = MIN (real, stream->bound_end);
-	real = MAX (real, stream->bound_start);
+	/* sanity check the resultant offset */
+	if (real < stream->bound_start)
+		return -1;
 	
-	if (real != stream->position && mstream->eos)
+	if (stream->bound_end != -1 && real > stream->bound_end)
+		return -1;
+	
+	/* reset eos if appropriate */
+	if ((stream->bound_end != -1 && real < stream->bound_end) ||
+	    (mstream->eos && real < stream->position))
 		mstream->eos = FALSE;
 	
 	stream->position = real;
