@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -151,7 +148,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
  *
  * Attempts to read up to @len bytes from @stream into @buf.
  *
- * Returns the number of bytes read or -1 on fail.
+ * Returns the number of bytes read or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_read (GMimeStream *stream, char *buf, size_t len)
@@ -182,7 +179,7 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
  *
  * Attempts to write up to @len bytes of @buf to @stream.
  *
- * Returns the number of bytes written or -1 on fail.
+ * Returns the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_write (GMimeStream *stream, const char *buf, size_t len)
@@ -211,7 +208,7 @@ stream_flush (GMimeStream *stream)
  *
  * Sync's the stream to disk.
  *
- * Returns 0 on success or -1 on fail.
+ * Returns %0 on success or %-1 on fail.
  **/
 int
 g_mime_stream_flush (GMimeStream *stream)
@@ -236,7 +233,7 @@ stream_close (GMimeStream *stream)
  *
  * Closes the stream.
  *
- * Returns 0 on success or -1 on fail.
+ * Returns %0 on success or %-1 on fail.
  **/
 int
 g_mime_stream_close (GMimeStream *stream)
@@ -279,7 +276,6 @@ static int
 stream_reset (GMimeStream *stream)
 {
 	d(g_warning ("Invoked default stream_reset implementation."));
-	stream->position = stream->bound_start;
 	return 0;
 }
 
@@ -290,14 +286,19 @@ stream_reset (GMimeStream *stream)
  *
  * Resets the stream.
  *
- * Returns 0 on success or -1 on fail.
+ * Returns %0 on success or %-1 on fail.
  **/
 int
 g_mime_stream_reset (GMimeStream *stream)
 {
+	int rv;
+	
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return GMIME_STREAM_GET_CLASS (stream)->reset (stream);
+	if ((rv = GMIME_STREAM_GET_CLASS (stream)->reset (stream)) == 0)
+		stream->position = stream->bound_start;
+	
+	return rv;
 }
 
 
@@ -319,15 +320,16 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
  * the argument @offset according to the
  * directive @whence as follows:
  *
- *     GMIME_STREAM_SEEK_SET: The offset is set to @offset bytes.
+ *     #GMIME_STREAM_SEEK_SET: Seek @offset bytes relative to
+ *     the beginning (bound_start) of the stream.
  *
- *     GMIME_STREAM_SEEK_CUR: The offset is set to its current
- *     location plus @offset bytes.
+ *     #GMIME_STREAM_SEEK_CUR: Seek @offset bytes relative to the
+ *     current offset of the stream.
  *
- *     GMIME_STREAM_SEEK_END: The offset is set to the size of the
- *     stream plus @offset bytes.
+ *     #GMIME_STREAM_SEEK_END: Seek @offset bytes relative to the
+ *     end of the stream (bound_end if non-negative).
  *
- * Returns the resultant position on success or -1 on fail.
+ * Returns the resultant position on success or %-1 on fail.
  **/
 off_t
 g_mime_stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
@@ -352,7 +354,7 @@ stream_tell (GMimeStream *stream)
  *
  * Gets the current offset within the stream.
  *
- * Returns the current position within the stream or -1 on fail.
+ * Returns the current position within the stream or %-1 on fail.
  **/
 off_t
 g_mime_stream_tell (GMimeStream *stream)
@@ -377,7 +379,7 @@ stream_length (GMimeStream *stream)
  *
  * Gets the length of the stream.
  *
- * Returns the length of the stream or -1 on fail.
+ * Returns the length of the stream or %-1 if unknown.
  **/
 ssize_t
 g_mime_stream_length (GMimeStream *stream)
@@ -413,8 +415,7 @@ g_mime_stream_substream (GMimeStream *stream, off_t start, off_t end)
 	
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), NULL);
 	
-	sub = GMIME_STREAM_GET_CLASS (stream)->substream (stream, start, end);
-	if (sub) {
+	if ((sub = GMIME_STREAM_GET_CLASS (stream)->substream (stream, start, end))) {
 		sub->super_stream = stream;
 		g_object_ref (stream);
 	}
@@ -429,7 +430,7 @@ g_mime_stream_substream (GMimeStream *stream, off_t start, off_t end)
  *
  * Ref's a stream.
  *
- * WARNING: This method is deprecated. Use #g_object_ref instead.
+ * WARNING: This method is deprecated. Use g_object_ref() instead.
  **/
 void
 g_mime_stream_ref (GMimeStream *stream)
@@ -446,7 +447,7 @@ g_mime_stream_ref (GMimeStream *stream)
  *
  * Unref's a stream.
  *
- * WARNING: This method is deprecated. Use g_object_unref instead.
+ * WARNING: This method is deprecated. Use g_object_unref() instead.
  **/
 void
 g_mime_stream_unref (GMimeStream *stream)
@@ -487,7 +488,7 @@ g_mime_stream_set_bounds (GMimeStream *stream, off_t start, off_t end)
  *
  * Writes @string to @stream.
  *
- * Returns the number of bytes written or -1 on fail.
+ * Returns the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_write_string (GMimeStream *stream, const char *string)
@@ -507,7 +508,7 @@ g_mime_stream_write_string (GMimeStream *stream, const char *string)
  *
  * Write formatted output to a stream.
  *
- * Returns the number of bytes written or -1 on fail.
+ * Returns the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_printf (GMimeStream *stream, const char *fmt, ...)
@@ -540,7 +541,7 @@ g_mime_stream_printf (GMimeStream *stream, const char *fmt, ...)
  *
  * Attempts to write stream @src to stream @dest.
  *
- * Returns the number of bytes written or -1 on fail.
+ * Returns the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
@@ -552,8 +553,7 @@ g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
 	g_return_val_if_fail (GMIME_IS_STREAM (dest), -1);
 	
 	while (!g_mime_stream_eos (src)) {
-		nread = g_mime_stream_read (src, buf, sizeof (buf));
-		if (nread < 0)
+		if ((nread = g_mime_stream_read (src, buf, sizeof (buf))) < 0)
 			return -1;
 		
 		if (nread > 0) {
@@ -561,8 +561,7 @@ g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
 			while (nwritten < nread) {
 				ssize_t len;
 				
-				len = g_mime_stream_write (dest, buf + nwritten, nread - nwritten);
-				if (len < 0)
+				if ((len = g_mime_stream_write (dest, buf + nwritten, nread - nwritten)) < 0)
 					return -1;
 				
 				nwritten += len;
@@ -584,7 +583,7 @@ g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
  *
  * Writes at most @count blocks described by @vector to @stream.
  *
- * Returns the number of bytes written or -1 on fail.
+ * Returns the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_writev (GMimeStream *stream, GMimeStreamIOVector *vector, size_t count)
