@@ -1,9 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Michael Zucchi <notzed@ximian.com>
- *           Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -29,10 +25,7 @@
 #include <time.h>
 #include <stdarg.h>
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 typedef enum {
 	GMIME_PART_ENCODING_DEFAULT,
@@ -55,13 +48,35 @@ typedef struct _GMimeReferences GMimeReferences;
 #define BASE64_ENCODE_LEN(x) ((size_t) ((x) * 5 / 3) + 4)  /* conservative would be ((x * 4 / 3) + 4) */
 #define QP_ENCODE_LEN(x)     ((size_t) ((x) * 7 / 2) + 4)  /* conservative would be ((x * 3) + 4) */
 
+
+/**
+ * GMIME_UUEDECODE_STATE_INIT:
+ *
+ * Initial state for the g_mime_utils_uudecode_step() function.
+ **/
 #define GMIME_UUDECODE_STATE_INIT   (0)
+
+
+/**
+ * GMIME_UUEDECODE_STATE_BEGIN:
+ *
+ * State for the g_mime_utils_uudecode_step() function, denoting that
+ * the 'begin' line has been found.
+ **/
 #define GMIME_UUDECODE_STATE_BEGIN  (1 << 16)
+
+
+/**
+ * GMIME_UUEDECODE_STATE_END:
+ *
+ * State for the g_mime_utils_uudecode_step() function, denoting that
+ * the end of the UU encoded block has been found.
+ **/
 #define GMIME_UUDECODE_STATE_END    (1 << 17)
 #define GMIME_UUDECODE_STATE_MASK   (GMIME_UUDECODE_STATE_BEGIN | GMIME_UUDECODE_STATE_END)
 
-time_t g_mime_utils_header_decode_date (const char *in, int *saveoffset);
-char  *g_mime_utils_header_format_date (time_t time, int offset);
+time_t g_mime_utils_header_decode_date (const char *in, int *tz_offset);
+char  *g_mime_utils_header_format_date (time_t date, int tz_offset);
 
 char *g_mime_utils_generate_message_id (const char *fqdn);
 
@@ -115,8 +130,6 @@ size_t g_mime_utils_quoted_encode_step (const unsigned char *in, size_t inlen, u
 size_t g_mime_utils_quoted_encode_close (const unsigned char *in, size_t inlen, unsigned char *out, int *state, int *save);
 
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GMIME_UTILS_H__ */
